"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RouteGenerator = void 0;
const glob_1 = require("glob");
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const promises_1 = require("fs/promises");
/**
 * 路由生成器类
 */
class RouteGenerator {
    constructor(options) {
        this.options = {
            scanDir: 'src/pages/**/*.vue',
            outputDir: 'src/router/',
            exclude: [],
            layoutPath: '@/pages/layout/index.vue',
            ...options
        };
    }
    /**
     * 生成路由配置
     */
    async generate() {
        try {
            const files = await this.scanFiles();
            if (!files.length) {
                console.warn('No .vue files found in the specified directory');
                return;
            }
            const routes = this.generateRoutes(files);
            const imports = this.generateImports(files);
            await this.writeRouteFile(routes, imports);
        }
        catch (error) {
            console.error('Failed to generate routes:', error);
            throw error; // 向上传播错误
        }
    }
    /**
     * 扫描文件
     * @returns 匹配的文件路径数组
     */
    async scanFiles() {
        try {
            const files = await (0, glob_1.glob)(this.options.scanDir, {
                ignore: [...this.options.exclude, '**/node_modules/**'],
                absolute: true,
                nodir: true
            });
            return files;
        }
        catch (error) {
            console.error('Failed to scan files:', error);
            return [];
        }
    }
    /**
     * 生成导入声明
     * @param files 文件路径数组
     * @returns 导入声明数组
     */
    generateImports(files) {
        return files.map(file => ({
            name: this.getComponentName(file),
            path: this.getImportPath(file)
        }));
    }
    /**
     * 生成路由配置
     * @param files 文件路径数组
     * @returns 路由节点数组
     */
    generateRoutes(files) {
        const fileGroups = this.groupFilesByDirectory(files);
        return Object.entries(fileGroups).map(([dir, files]) => ({
            path: this.getRoutePath(dir),
            component: 'Layout',
            name: this.getRouteBaseName(dir),
            children: files.map(file => ({
                path: this.getSubRoutePath(file),
                component: this.getComponentName(file),
                name: this.getRouteName(dir, file)
            }))
        }));
    }
    /**
     * 写入路由文件
     * @param routes 路由配置
     * @param imports 导入声明
     */
    async writeRouteFile(routes, imports) {
        const content = this.generateRouteFileContent(routes, imports);
        try {
            await (0, promises_1.mkdir)(this.options.outputDir, { recursive: true });
            const outputPath = path_1.default.join(this.options.outputDir, 'routes.ts');
            await fs_1.promises.writeFile(outputPath, content, 'utf-8');
        }
        catch (error) {
            console.error('Failed to write route file:', error);
        }
    }
    /**
     * 获取组件名称
     * @param file 文件路径
     * @returns 组件名称
     */
    getComponentName(file) {
        const basename = path_1.default.basename(file, '.vue');
        return basename.charAt(0).toUpperCase() + basename.slice(1);
    }
    /**
     * 获取导入路径
     * @param file 文件路径
     * @returns 导入路径
     */
    getImportPath(file) {
        return file.replace(/^src\//, '@/');
    }
    /**
     * 获取路由路径
     * @param dir 目录名
     * @returns 路由路径
     */
    getRoutePath(dir) {
        return '/' + dir.toLowerCase().replace(/\\/g, '/');
    }
    /**
     * 获取子路由路径
     * @param file 文件路径
     * @returns 子路由路径
     */
    getSubRoutePath(file) {
        const basename = path_1.default.basename(file, '.vue');
        return basename === 'index' ? '' : basename.toLowerCase();
    }
    /**
     * 获取路由基础名称
     * @param dir 目录名
     * @returns 路由基础名称
     */
    getRouteBaseName(dir) {
        return dir.split('_')
            .map(part => part.charAt(0).toUpperCase() + part.slice(1))
            .join('');
    }
    /**
     * 获取路由名称
     * @param dir 目录名
     * @param file 文件路径
     * @returns 路由名称
     */
    getRouteName(dir, file) {
        const dirName = dir.split('/').pop() || '';
        const fileName = path_1.default.basename(file, '.vue');
        const baseName = this.getRouteBaseName(dirName);
        return fileName === 'index'
            ? baseName
            : baseName + fileName.charAt(0).toUpperCase() + fileName.slice(1);
    }
    /**
     * 按目录分组文件
     * @param files 文件路径数组
     * @returns 分组后的文件映射
     */
    groupFilesByDirectory(files) {
        return files.reduce((acc, file) => {
            const dir = path_1.default.dirname(file).split('/pages/')[1];
            if (!acc[dir])
                acc[dir] = [];
            acc[dir].push(file);
            return acc;
        }, {});
    }
    /**
     * 生成路由文件内容
     * @param routes 路由配置
     * @param imports 导入声明
     * @returns 生成的文件内容
     */
    generateRouteFileContent(routes, imports) {
        const routeStr = JSON.stringify(routes, null, 2)
            .replace(/"component": "([^"]+)"/g, 'component: $1')
            .replace(/"children": /g, 'children: ')
            .replace(/"path": /g, 'path: ')
            .replace(/"name": /g, 'name: ');
        return `
import type { RouteRecordRaw } from 'vue-router'
import Layout from '${this.options.layoutPath}'

${imports.map(imp => `const ${imp.name} = () => import('${imp.path}')`).join('\n')}

const Router: Array<RouteRecordRaw> = ${routeStr}

export default Router
`;
    }
}
exports.RouteGenerator = RouteGenerator;
//# sourceMappingURL=generator.js.map